#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "nrsrc/nr.h"
#include "nrsrc/nrutil.h"

#include "prototypes.h"
#include "globvars.h"



int main(int argc, char *argv[])
{
  char parameter_filename[100];
  char buf[1000];
  FILE *fd;
  int rep;

  if(argc != 2)
    {
      fprintf(stderr, "\n\nwrong argument(s).  Specify a parameterfile.\n\n");
      exit(0);
    }
  strcpy(parameter_filename, argv[1]);

  read_parameterfile(parameter_filename);

  init_units();			/* set system of units */

  structure();			/* determine structure of halo, disk, and bulge */

  init();			/* allocate arrays */

  init_clouds();


  if(M_DISK > 0)
    {
      init_central_densities();
      
      integrate_gasdensity();
      
      determine_cumulative_gasdensity();
      

      for(rep = 0; rep < 5; rep++)
	{
	  set_dummy_particles();
	  
	  compute_vertical_force_field();
	  
	  integrate_and_adjust();
	}
      
      dump_gas_density();
    }

  compute_radial_force_field();

  compute_vstream_gas();
  compute_velocity_dispersions_disk();
  compute_velocity_dispersions_halo();
  compute_velocity_dispersions_bulge();

  set_gas_positions();
  set_halo_positions();
  set_disk_positions();
  set_bulge_positions();

  set_halo_velocities();
  set_disk_velocities();
  set_gas_velocities();
  set_bulge_velocities();


  dump_veldisp_field();

  mkdir(OutputDir, 02755);

  sprintf(buf, "%s/%s", OutputDir, OutputFile);

  save_particles(buf);

  sprintf(buf, "%s/curve.txt", OutputDir);

  if((fd = fopen(buf, "w")))
    {
      printf("writing circular velocity curve + Toomre's Q\n");
      write_header(fd);
      plot_circular_speeds(fd);
      plot_toomre_stability(fd);
      fclose(fd);
    }
  else
    {
      fprintf(stderr, "Can't open file '%s'.\n", buf);
      exit(0);
    }
  printf("done.\n");


  printf("Disk scale length: %g\n", H);
  printf("R200: %g\n", R200);

  return 0;
}



void write_header(FILE * fd)
{
  fprintf(fd, "%g\n", CC);
  fprintf(fd, "%g\n", V200);
  fprintf(fd, "%g\n", LAMBDA);
  fprintf(fd, "%g\n", MD);
  fprintf(fd, "%g\n", JD);
  fprintf(fd, "%g\n", MB);
  fprintf(fd, "%g\n", DiskHeight);
  fprintf(fd, "%g\n", BulgeSize);
  fprintf(fd, "\n%g\n", R200);
  fprintf(fd, "%g\n\n", H);
}
